@@ -5,17 +5,30 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2, incorporated herein by reference.
 
-import re
+from __future__ import absolute_import
+
 import binascii
+import re
 
-from PyQt4.QtCore import *
-from PyQt4.QtGui import *
+from .qtcore import (
+    QSize,
+    Qt,
+    pyqtSignal,
+)
+from .qtgui import (
+    QHBoxLayout,
+    QLabel,
+    QWidget,
+)
 
-from mercurial import error
+from mercurial import (
+    error,
+    pycompat,
+)
 
-from tortoisehg.util import hglib
-from tortoisehg.util.i18n import _
-from tortoisehg.hgqt import qtlib
+from ..util import hglib
+from ..util.i18n import _
+from . import qtlib
 
 PANEL_DEFAULT = ('rev', 'summary', 'user', 'dateage', 'branch', 'close',
                  'tags', 'graft', 'transplant', 'obsolete',
@@ -84,7 +97,7 @@ class Factory(object):
         if 'type' not in style:
             raise _("must be specified 'type' in style")
         type = style['type']
-        assert type in ('panel', 'label')
+        assert type in ('panel', 'label'), type
 
         # create widget
         args = (target, style, custom, repo, self.info)
@@ -103,6 +116,7 @@ class UnknownItem(Exception):
 class SummaryInfo(object):
 
     LABELS = {'rev': _('Revision:'), 'revnum': _('Revision:'),
+              'gitcommit': _('Git Commit:'),
               'revid': _('Revision:'), 'summary': _('Summary:'),
               'user': _('User:'), 'date': _('Date:'),'age': _('Age:'),
               'dateage': _('Date:'), 'branch': _('Branch:'),
@@ -128,17 +142,19 @@ class SummaryInfo(object):
                 revnum = self.get_data('revnum', *args)
                 revid = self.get_data('revid', *args)
                 if revid:
-                    return (revnum, revid)
+                    return revnum, revid
                 return None
             elif item == 'revnum':
                 return ctx.rev()
             elif item == 'revid':
                 return str(ctx)
+            elif item == 'gitcommit':
+                return hglib.gitcommit(ctx)
             elif item == 'desc':
-                return hglib.tounicode(ctx.description().replace('\0', ''))
+                return hglib.tounicode(ctx.description().replace(b'\0', b''))
             elif item == 'summary':
                 summary = hglib.longsummary(
-                    ctx.description().replace('\0', ''))
+                    ctx.description().replace(b'\0', b''))
                 if len(summary) == 0:
                     return None
                 return summary
@@ -153,12 +169,12 @@ class SummaryInfo(object):
                 date = self.get_data('date', *args)
                 age = self.get_data('age', *args)
                 if date and age:
-                    return (date, age)
+                    return date, age
                 return None
             elif item == 'date':
                 date = ctx.date()
                 if date:
-                    return hglib.displaytime(date)
+                    return hglib.tounicode(hglib.displaytime(date))
                 return None
             elif item == 'age':
                 date = ctx.date()
@@ -166,7 +182,7 @@ class SummaryInfo(object):
                     return hglib.age(date).decode('utf-8')
                 return None
             elif item == 'rawbranch':
-                return ctx.branch() or None
+                return hglib.tounicode(ctx.branch()) or None
             elif item == 'branch':
                 value = self.get_data('rawbranch', *args)
                 if value:
@@ -182,22 +198,22 @@ class SummaryInfo(object):
                     return value
                 return None
             elif item == 'close':
-                return ctx.extra().get('close')
+                return hglib.tounicode(ctx.extra().get(b'close'))
             elif item == 'tags':
-                return ctx.thgtags() or None
+                return [hglib.tounicode(tag) for tag in ctx.thgtags()] or None
             elif item == 'graft':
                 extra = ctx.extra()
                 try:
-                    return extra['source']
+                    return hglib.tounicode(extra[b'source'])
                 except KeyError:
                     pass
                 return None
             elif item == 'transplant':
                 extra = ctx.extra()
                 try:
-                    ts = extra['transplant_source']
+                    ts = extra[b'transplant_source']
                     if ts:
-                        return binascii.hexlify(ts)
+                        return hglib.tounicode(binascii.hexlify(ts))
                 except KeyError:
                     pass
                 return None
@@ -207,34 +223,33 @@ class SummaryInfo(object):
                     obsoletestate.append('obsolete')
                 if ctx.extinct():
                     obsoletestate.append('extinct')
-                obsoletestate += ctx.troubles()
+                obsoletestate += map(hglib.tounicode, ctx.instabilities())
                 if obsoletestate:
                     return obsoletestate
                 return None
             elif item == 'p4':
                 extra = ctx.extra()
-                p4cl = extra.get('p4', None)
-                return p4cl and ('changelist %s' % p4cl)
+                p4cl = extra.get(b'p4', None)
+                return p4cl and hglib.tounicode(b'changelist %s' % p4cl)
             elif item == 'svn':
                 extra = ctx.extra()
-                cvt = extra.get('convert_revision', '')
-                if cvt.startswith('svn:'):
-                    result = cvt.split('/', 1)[-1]
+                cvt = extra.get(b'convert_revision', b'')
+                if cvt.startswith(b'svn:'):
+                    result = cvt.split(b'/', 1)[-1]
                     if cvt != result:
-                        return result
-                    return cvt.split('@')[-1]
+                        return hglib.tounicode(result)
+                    return hglib.tounicode(cvt.split(b'@')[-1])
                 else:
                     return None
             elif item == 'converted':
                 extra = ctx.extra()
-                cvt = extra.get('convert_revision', '')
-                if cvt and not cvt.startswith('svn:'):
-                    return cvt
+                cvt = extra.get(b'convert_revision', b'')
+                if cvt and not cvt.startswith(b'svn:'):
+                    return hglib.tounicode(cvt)
                 else:
                     return None
             elif item == 'ishead':
-                childbranches = [cctx.branch() for cctx in ctx.children()]
-                return ctx.branch() not in childbranches
+                return ctx.node() in ctx.repo().branchheads(ctx.branch())
             elif item == 'mqoriginalparent':
                 target = ctx.thgmqoriginalparent()
                 if not target:
@@ -244,7 +259,7 @@ class SummaryInfo(object):
                     return None
                 if target not in ctx._repo:
                     return None
-                return target
+                return hglib.tounicode(target)
             raise UnknownItem(item)
         if 'data' in custom and not kargs.get('usepreset', False):
             try:
@@ -288,13 +303,14 @@ class SummaryInfo(object):
                 if revnum is not None and revid is not None:
                     return '%s (%s)' % (revnum, revid)
                 return '%s' % revid
-            elif item in ('revid', 'graft', 'transplant', 'mqoriginalparent'):
+            elif item in ('revid', 'gitcommit', 'graft', 'transplant',
+                          'mqoriginalparent'):
                 return qtlib.markup(value, **mono)
             elif item in ('revnum', 'p4', 'close', 'converted'):
                 return str(value)
             elif item == 'svn':
                 # svn is always in utf-8 because ctx.extra() isn't converted
-                return unicode(value, 'utf-8', 'replace')
+                return pycompat.unicode(value, 'utf-8', 'replace')
             elif item in ('rawbranch', 'branch'):
                 opts = dict(fg='black', bg='#aaffaa')
                 return qtlib.markup(' %s ' % value, **opts)
@@ -330,7 +346,7 @@ class SummaryInfo(object):
     def get_widget(self, item, widget, ctx, custom, **kargs):
         args = (widget, ctx, custom)
         def default_func(widget, item, markups):
-            if isinstance(markups, basestring):
+            if hglib.isbasestring(markups):
                 markups = (markups,)
             labels = []
             for text in markups:
@@ -351,14 +367,11 @@ class SummaryInfo(object):
 class SummaryBase(object):
 
     def __init__(self, target, custom, repo, info):
-        if target is None:
-            self.target = None
-        else:
-            self.target = str(target)
+        self.target = target
         self.custom = custom
         self.repo = repo
         self.info = info
-        self.ctx = repo.changectx(self.target)
+        self.update_ctx()
 
     def get_data(self, item, **kargs):
         return self.info.get_data(item, self, self.ctx, self.custom, **kargs)
@@ -377,10 +390,7 @@ class SummaryBase(object):
 
     def update(self, target=None, custom=None, repo=None):
         self.ctx = None
-        if target is None:
-            target = self.target
         if target is not None:
-            target = str(target)
             self.target = target
         if custom is not None:
             self.custom = custom
@@ -389,7 +399,14 @@ class SummaryBase(object):
         if repo is not None:
             self.repo = repo
         if self.ctx is None:
-            self.ctx = repo.changectx(target)
+            self.update_ctx()
+
+    def update_ctx(self):
+        if isinstance(self.target, pycompat.unicode):
+            target = hglib.fromunicode(self.target)
+        else:
+            target = self.target
+        self.ctx = self.repo[target]
 
 PANEL_TMPL = '<tr><td style="padding-right:6px">%s</td><td>%s</td></tr>'
 
@@ -404,7 +421,7 @@ class SummaryPanel(SummaryBase, QWidget):
         self.csstyle = style
 
         hbox = QHBoxLayout()
-        hbox.setMargin(0)
+        hbox.setContentsMargins(0, 0, 0, 0)
         hbox.setSpacing(0)
         self.setLayout(hbox)
         self.revlabel = None
@@ -413,21 +430,24 @@ class SummaryPanel(SummaryBase, QWidget):
     def update(self, target=None, style=None, custom=None, repo=None):
         SummaryBase.update(self, target, custom, repo)
 
+        layout = self.layout()
+        assert isinstance(layout, QHBoxLayout)
+
         if style is not None:
             self.csstyle = style
 
         if self.revlabel is None:
             self.revlabel = QLabel()
             self.revlabel.linkActivated.connect(self.linkActivated)
-            self.layout().addWidget(self.revlabel, 0, Qt.AlignTop)
+            layout.addWidget(self.revlabel, 0, Qt.AlignTop)
 
         if 'expandable' in self.csstyle and self.csstyle['expandable']:
             if self.expand_btn.parentWidget() is None:
                 self.expand_btn.clicked.connect(lambda: self.update())
                 margin = QHBoxLayout()
-                margin.setMargin(3)
+                margin.setContentsMargins(3, 3, 3, 3)
                 margin.addWidget(self.expand_btn, 0, Qt.AlignTop)
-                self.layout().insertLayout(0, margin)
+                layout.insertLayout(0, margin)
             self.expand_btn.setVisible(True)
         elif self.expand_btn.parentWidget() is not None:
             self.expand_btn.setHidden(True)
@@ -447,7 +467,7 @@ class SummaryPanel(SummaryBase, QWidget):
 
         if 'margin' in self.csstyle:
             margin = self.csstyle['margin']
-            assert isinstance(margin, (int, long))
+            assert isinstance(margin, (int, pycompat.long)), repr(margin)
             buf = '<table style="margin: %spx">' % margin
         else:
             buf = '<table>'
@@ -457,7 +477,7 @@ class SummaryPanel(SummaryBase, QWidget):
             if not markups:
                 continue
             label = qtlib.markup(self.get_label(item), weight='bold')
-            if isinstance(markups, basestring):
+            if hglib.isbasestring(markups):
                 markups = [markups,]
             buf += PANEL_TMPL % (label, markups.pop(0))
             for markup in markups:
@@ -520,7 +540,7 @@ class SummaryLabel(SummaryBase, QLabel):
                 markups = self.get_markup(item)
                 if not markups:
                     continue
-                if isinstance(markups, basestring):
+                if hglib.isbasestring(markups):
                     markups = (markups,)
                 data[item] = ', '.join(markups)
             if len(data) == 0:
