@@ -34,6 +34,7 @@ from .qtgui import (
 
 from mercurial import (
     error,
+    pycompat,
     repoview,
     scmutil,
     util,
@@ -54,7 +55,7 @@ def _firstword(query):
     lquery = hglib.fromunicode(query)
     try:
         for token, value, _pos in hglib.tokenizerevspec(lquery):
-            if token == 'symbol' or token == 'string':
+            if token == b'symbol' or token == b'string':
                 return value  # localstr
     except error.ParseError:
         pass
@@ -64,7 +65,7 @@ def _querytype(repo, query):
     >>> # TODO: maybe replace with real repo
     >>> origisrevsymbol = scmutil.isrevsymbol
     >>> scmutil.isrevsymbol = lambda repo, changeid: changeid in repo
-    >>> repo = set('0 1 2 3 . stable'.split())
+    >>> repo = set(b'0 1 2 3 . stable'.split())
     >>> _querytype(repo, u'') is None
     True
     >>> _querytype(repo, u'quick fox')
@@ -293,18 +294,18 @@ class RepoFilterBar(QToolBar):
         self.runQuery()
 
     def _prepareQuery(self):
-        query = unicode(self.revsetcombo.currentText()).strip()
+        query = pycompat.unicode(self.revsetcombo.currentText()).strip()
         if _querytype(self._repo, query) == 'keyword':
             return hglib.formatrevspec('keyword(%s)', query)
         else:
             return query
 
     def _isUnsavedQuery(self):
-        return unicode(self.revsetcombo.currentText()).startswith(' ')
+        return pycompat.unicode(self.revsetcombo.currentText()).startswith(' ')
 
     @pyqtSlot()
     def _updateQueryType(self):
-        query = unicode(self.revsetcombo.currentText()).strip()
+        query = pycompat.unicode(self.revsetcombo.currentText()).strip()
         qtype = _querytype(self._repo, query)
         if not qtype:
             self._revsettypelabel.hide()
@@ -459,17 +460,17 @@ class RepoFilterBar(QToolBar):
         if self._abranchAction.isChecked():
             branches = sorted(set([self._repo[n].branch()
                 for n in self._repo.heads()
-                if not self._repo[n].extra().get('close')]))
+                if not self._repo[n].extra().get(b'close')]))
         elif self._cbranchAction.isChecked():
             branches = sorted(self._repo.branchmap())
         else:
             branches = hglib.namedbranches(self._repo)
 
         # easy access to common branches (Python sorted() is stable)
-        priomap = {self._repo.dirstate.branch(): -2, 'default': -1}
+        priomap = {self._repo.dirstate.branch(): -2, b'default': -1}
         branches = sorted(branches, key=lambda e: priomap.get(e, 0))
 
-        branches = map(hglib.tounicode, branches)
+        branches = pycompat.maplist(hglib.tounicode, branches)
 
         self._branchCombo.blockSignals(True)
         self._branchCombo.clear()
@@ -497,7 +498,7 @@ class RepoFilterBar(QToolBar):
         """Return the current branch name [unicode]"""
         index = self._branchCombo.currentIndex()
         branch = self._branchCombo.itemData(index)
-        return unicode(branch)
+        return pycompat.unicode(branch)
 
     def branchAncestorsIncluded(self):
         return self._allparAction.isChecked()
@@ -525,5 +526,5 @@ class RepoFilterBar(QToolBar):
         self._updateShowHiddenBtnState()
 
     def _updateShowHiddenBtnState(self):
-        hashidden = bool(repoview.filterrevs(self._repo, 'visible'))
+        hashidden = bool(repoview.filterrevs(self._repo, b'visible'))
         self.showHiddenBtn.setEnabled(hashidden)
