@@ -38,6 +38,7 @@ from .qtgui import (
 
 from mercurial import (
     error,
+    pycompat,
 )
 from mercurial.utils import (
     dateutil,
@@ -70,7 +71,8 @@ UtcDateColumn = 11
 ChangesColumn = 12
 ConvertedColumn = 13
 PhaseColumn = 14
-FileColumn = 15
+TopicColumn = 15
+FileColumn = 16
 
 COLUMNHEADERS = (
     ('Graph', _('Graph', 'column header')),
@@ -88,6 +90,7 @@ COLUMNHEADERS = (
     ('Changes', _('Changes', 'column header')),
     ('Converted', _('Converted From', 'column header')),
     ('Phase', _('Phase', 'column header')),
+    ('Topic', _('Topic', 'column header')),
     ('Filename', _('Filename', 'column header')),
     )
 ALLCOLUMNS = tuple(name for name, _text in COLUMNHEADERS)
@@ -108,7 +111,7 @@ def _parsebranchcolors(value):
     [('foo bar', 'black'), ('foo:bar', 'white')]
 
     >>> _parsebranchcolors(r'\u00c0:black')
-    [('\xc0', 'black')]
+    [(u'\xc0', 'black')]
     >>> _parsebranchcolors('\xc0:black')
     [('\xc0', 'black')]
 
@@ -131,9 +134,10 @@ def _parsebranchcolors(value):
         key = key.replace('\\:', ':').replace('\\ ', ' ')
         if r'\u' in key:
             # apply unicode_escape only if \u found, so that raw non-ascii
-            # value isn't always mangled.
+            # value isn't always mangled. leave non-ascii string unmodified
+            # since we don't know how to feed it to unicode_escape decoder.
             try:
-                key = hglib.fromunicode(key.decode('unicode_escape'))
+                key = key.encode('ascii').decode('unicode_escape')
             except (UnicodeDecodeError, UnicodeEncodeError):
                 continue
         colors.append((key, val))
@@ -153,7 +157,7 @@ class HgRepoListModel(QAbstractTableModel):
     _defaultcolumns = ('Graph', 'Rev', 'Branch', 'Description', 'Author',
                        'Age', 'Tags', 'Phase')
 
-    _mqtags = ('qbase', 'qtip', 'qparent')
+    _mqtags = (b'qbase', b'qtip', b'qparent')
 
     def __init__(self, repoagent, parent=None):
         QAbstractTableModel.__init__(self, parent)
@@ -197,7 +201,7 @@ class HgRepoListModel(QAbstractTableModel):
 
         # Set the colors specified in the tortoisehg.brachcolors config key
         self._branch_colors.update(_parsebranchcolors(
-            self.repo.ui.config('tortoisehg', 'branchcolors')))
+            self._repoagent.configString('tortoisehg', 'branchcolors')))
 
     def setBranch(self, branch, allparents=False):
         branchchanged = (branch != self._filterbranch)
@@ -222,16 +226,16 @@ class HgRepoListModel(QAbstractTableModel):
             return graph.Graph(self.repo, [])  # no matches found
         if self._selectedrevs and self._filterbyrevset:
             opts['revset'] = self._selectedrevs
-            opts['showfamilyline'] = \
-                self.repo.ui.configbool('tortoisehg', 'showfamilyline', True)
+            opts['showfamilyline'] = self._repoagent.configBool(
+                'tortoisehg', 'showfamilyline')
             grapher = graph.revision_grapher(self.repo, opts)
-            if self.repo.ui.configbool('tortoisehg', 'graphopt', False):
+            if self._repoagent.configBool('tortoisehg', 'graphopt'):
                 return graphopt.Graph(self.repo, opts)
             return graph.Graph(self.repo, grapher)
         else:
             opts['allparents'] = self._allparents
             grapher = graph.revision_grapher(self.repo, opts)
-            if self.repo.ui.configbool('tortoisehg', 'graphopt', False):
+            if self._repoagent.configBool('tortoisehg', 'graphopt'):
                 g = graphopt.Graph(self.repo, opts)
             else:
                 g = graph.Graph(self.repo, grapher)
@@ -293,7 +297,7 @@ class HgRepoListModel(QAbstractTableModel):
                 self._shrinkRowCount()  # avoid further exceptions at data()
                 raise
             self._expandRowCount()  # old rows may be mapped to inserted rows
-            for rev, ois in oldindexmap.iteritems():
+            for rev, ois in oldindexmap.items():
                 try:
                     row = self.graph.index(rev)
                 except ValueError:
@@ -319,7 +323,7 @@ class HgRepoListModel(QAbstractTableModel):
         return self._revspec
 
     def setRevset(self, revspec):
-        revspec = unicode(revspec)
+        revspec = pycompat.unicode(revspec)
         if revspec == self._revspec:
             return
         self._revspec = revspec
@@ -345,7 +349,7 @@ class HgRepoListModel(QAbstractTableModel):
             # new query is already running
             return
         if ret == 0:
-            revs = map(int, str(sess.readAll()).splitlines())
+            revs = pycompat.maplist(int, bytes(sess.readAll()).splitlines())
             if revs:
                 self.showMessage.emit(_('%d matches found') % len(revs))
             else:
@@ -372,10 +376,13 @@ class HgRepoListModel(QAbstractTableModel):
             self._rebuildGraph()
 
     def _reloadConfig(self):
-        _ui = self.repo.ui
-        self._fill_step = int(_ui.config('tortoisehg', 'graphlimit', 500))
-        self._authorcolor = _ui.configbool('tortoisehg', 'authorcolor')
-        self._fullauthorname = _ui.configbool('tortoisehg', 'fullauthorname')
+        self._fill_step = self._repoagent.configInt('tortoisehg', 'graphlimit')
+        self._authorcolor = self._repoagent.configBool(
+            'tortoisehg', 'authorcolor')
+        self._fullauthorname = self._repoagent.configBool(
+            'tortoisehg', 'fullauthorname')
+        self._show_branch_head_label = self._repoagent.configBool(
+            'tortoisehg', 'show-branch-head-label')
 
     @pyqtSlot()
     def _invalidate(self):
@@ -526,7 +533,7 @@ class HgRepoListModel(QAbstractTableModel):
         # example, and result in RevlogError. (issue #429)
         try:
             return self._safedata(index, role)
-        except error.RevlogError, e:
+        except error.RevlogError as e:
             if 'THGDEBUG' in os.environ:
                 raise
             if role == Qt.DisplayRole:
@@ -539,7 +546,8 @@ class HgRepoListModel(QAbstractTableModel):
         graphlen = len(self.graph)
         cachelen = len(self._cache)
         if graphlen > cachelen:
-            self._cache.extend({} for _i in xrange(graphlen - cachelen))
+            self._cache.extend({} for _i in
+                               pycompat.xrange(graphlen - cachelen))
         data = self._cache[row]
         idx = (role, index.column())
         if idx not in data:
@@ -565,7 +573,7 @@ class HgRepoListModel(QAbstractTableModel):
             if textfunc is None:
                 return None
             text = textfunc(self, ctx)
-            if not isinstance(text, unicode):
+            if not isinstance(text, pycompat.unicode):
                 text = hglib.tounicode(text)
             return text
         elif role == Qt.ForegroundRole:
@@ -623,23 +631,25 @@ class HgRepoListModel(QAbstractTableModel):
     def mimeData(self, indexes):
         data = set()
         for index in indexes:
-            row = str(index.row())
+            row = b'%d' % index.row()
             if row not in data:
                 data.add(row)
         qmd = QMimeData()
-        bytearray = QByteArray(','.join(sorted(data, reverse=True)))
+        bytearray = QByteArray(b','.join(sorted(data, reverse=True)))
         qmd.setData(mqpatchmimetype, bytearray)
         return qmd
 
     def dropMimeData(self, data, action, row, column, parent):
         if mqpatchmimetype not in data.formats():
             return False
-        dragrows = [int(r) for r in str(data.data(mqpatchmimetype)).split(',')]
+        dragrows = [int(r)
+                    for r in bytes(data.data(mqpatchmimetype)).split(b',')]
         destrow = parent.row()
         if destrow < 0:
             return False
-        unapplied = self.repo.thgmqunappliedpatches[::-1]
-        applied = [p.name for p in self.repo.mq.applied[::-1]]
+        unapplied = pycompat.maplist(hglib.tounicode,
+                                     self.repo.thgmqunappliedpatches[::-1])
+        applied = [hglib.tounicode(p.name) for p in self.repo.mq.applied[::-1]]
         if max(dragrows) >= len(unapplied):
             return False
         dragpatches = [unapplied[d] for d in dragrows]
@@ -650,7 +660,6 @@ class HgRepoListModel(QAbstractTableModel):
             destpatch = None  # next to working rev
 
         cmdline = hglib.buildcmdargs('qreorder', after=destpatch, *dragpatches)
-        cmdline = map(hglib.tounicode, cmdline)
         self._repoagent.runCommand(cmdline)
         return True
 
@@ -662,11 +671,12 @@ class HgRepoListModel(QAbstractTableModel):
         """Index that should be selected when the model is initially loaded
         or the row previously selected is gone"""
         repo = self.repo
-        initialsel = repo.ui.config('tortoisehg', 'initialrevision', 'current')
-        changeid = {'current': '.',
-                    'tip': 'tip',
+        initialsel = self._repoagent.configString(
+            'tortoisehg', 'initialrevision')
+        changeid = {'current': b'.',
+                    'tip': b'tip',
                     'workingdir': None,
-                    }.get(initialsel, '.')
+                    }.get(initialsel, b'.')
         rev = repo[changeid].rev()
         if self._selectedrevs and rev not in self._selectedrevs:
             rev = max(self._selectedrevs)
@@ -677,7 +687,7 @@ class HgRepoListModel(QAbstractTableModel):
         if self._filterbranch:
             # look for the first active revision as last ditch; should be
             # removed if filterbranch is merged with revset
-            for row in xrange(len(self.graph)):
+            for row in pycompat.xrange(len(self.graph)):
                 gnode = self.graph[row]
                 if not isinstance(gnode.rev, int):
                     continue
@@ -713,15 +723,15 @@ class HgRepoListModel(QAbstractTableModel):
             if rev in self._latesttags:
                 continue
             ctx = repo[rev]
-            tags = [t for t in ctx.tags()
-                    if repo.tagtype(t) and repo.tagtype(t) != 'local']
+            tags = [hglib.tounicode(t) for t in ctx.tags()
+                    if repo.tagtype(t) and repo.tagtype(t) != b'local']
             if tags:
                 self._latesttags[rev] = ctx.date()[0], 0, ':'.join(sorted(tags))
                 continue
             try:
                 # The tuples are laid out so the right one can be found by
                 # comparison.
-                if (ctx.parents()):
+                if ctx.parents():
                     pdate, pdist, ptag = max(
                         self._latesttags[p.rev()] for p in ctx.parents())
                 else:
@@ -737,12 +747,12 @@ class HgRepoListModel(QAbstractTableModel):
     def _gettags(self, ctx):
         if ctx.rev() is None:
             return ''
-        tags = [t for t in ctx.tags() if t not in self._mqtags]
-        return hglib.tounicode(','.join(tags))
+        tags = [hglib.tounicode(t) for t in ctx.tags() if t not in self._mqtags]
+        return ','.join(tags)
 
     def _getrev(self, ctx):
         rev = ctx.rev()
-        if type(rev) is int:
+        if isinstance(rev, int):
             return str(rev)
         elif rev is None:
             return u'%d+' % ctx.p1().rev()
@@ -765,7 +775,7 @@ class HgRepoListModel(QAbstractTableModel):
                 return u'\u2605 ' + _('Working Directory') + u' \u2605'
             else:
                 return '*** ' + _('Working Directory') + ' ***'
-        if self.repo.ui.configbool('tortoisehg', 'longsummary'):
+        if self._repoagent.configBool('tortoisehg', 'longsummary'):
             limit = 0x7fffffff  # unlimited (elide it by view)
         else:
             limit = None  # first line
@@ -784,13 +794,13 @@ class HgRepoListModel(QAbstractTableModel):
             self._branchheads[branch] = branchheads
 
         if ctx.rev() is None:
-            for pctx in ctx.parents():
-                if branchheads and pctx.node() not in branchheads:
-                    labels.append((_('Not a head revision!'), 'log.warning'))
+            if hglib.createsnewhead(ctx, branchheads):
+                labels.append((_('Creates new head!'), 'log.warning'))
             return labels
 
-        if ctx.node() in branchheads:
-            labels.append((hglib.tounicode(branch), 'log.branch'))
+        if self._show_branch_head_label:
+            if ctx.node() in branchheads:
+                labels.append((hglib.tounicode(branch), 'log.branch'))
 
         if ctx.thgmqunappliedpatch():
             style = 'log.unapplied_patch'
@@ -811,14 +821,16 @@ class HgRepoListModel(QAbstractTableModel):
                 style = 'log.tag'
             labels.append((hglib.tounicode(tag), style))
 
-        names = set(self.repo.ui.configlist('experimental', 'thg.displaynames'))
-        for name, ns in self.repo.names.iteritems():
-            if name not in names:
+        names = set(self._repoagent.configStringList(
+            'experimental', 'thg.displaynames'))
+        for name, ns in self.repo.names.items():
+            if pycompat.sysstr(name) not in names:
                 continue
             # we will use the templatename as the color name since those
             # two should be the same
             for name in ns.names(self.repo, ctx.node()):
-                labels.append((hglib.tounicode(name), 'log.%s' % ns.colorname))
+                labels.append((hglib.tounicode(name),
+                               'log.%s' % hglib.tounicode(ns.colorname)))
 
         return labels
 
@@ -860,6 +872,9 @@ class HgRepoListModel(QAbstractTableModel):
         except:
             return 'draft'
 
+    def _gettopic(self, ctx):
+        return hglib.tounicode(ctx.extra().get(b'topic'))
+
     def _hasFileColumn(self):
         return False # no FileColumn
 
@@ -889,6 +904,7 @@ class HgRepoListModel(QAbstractTableModel):
         UtcDateColumn: lambda self, ctx: hglib.utctime(ctx.date()),
         ConvertedColumn: _getconv,
         PhaseColumn: _getphase,
+        TopicColumn: _gettopic,
         }
 
 
@@ -920,7 +936,7 @@ class FileRevModel(HgRepoListModel):
         # as of Mercurial 2.6, workingfilectx.linkrev() does not work, and
         # this model has no virtual working-dir revision.
         if rev is None:
-            rev = '.'
+            rev = b'.'
         try:
             fctx = self.repo[rev][self._filename]
         except error.LookupError:
