@@ -48,10 +48,15 @@ from .qtgui import (
     QWidget,
 )
 
+from hgext.largefiles import (
+    lfutil,
+)
+
 from mercurial import (
     context,
     error,
     hg,
+    pycompat,
     scmutil,
     util,
 )
@@ -266,7 +271,7 @@ class StatusWidget(QWidget):
 
     def __set_defcheck(self, newdefcheck):
         if newdefcheck.lower() == 'amend':
-            newdefcheck = 'MAS'
+            newdefcheck = 'MARS'
         elif newdefcheck.lower() in ('commit', 'qnew', 'qrefresh'):
             newdefcheck = 'MAR!S'
         self._defcheck = newdefcheck
@@ -304,14 +309,16 @@ class StatusWidget(QWidget):
     def _updatePartials(self, fd):
         # remove files from the partials dictionary if they are not partial
         # selections, in order to simplify refresh.
+        model = self.tv.model()
+        assert model is not None
         dels = []
-        for file, oldchanges in self.partials.iteritems():
-            assert file in self.tv.model().checked
+        for file, oldchanges in self.partials.items():
+            assert file in model.checked, file
             if oldchanges.excludecount == 0:
-                self.tv.model().checked[file] = True
+                model.checked[file] = True
                 dels.append(file)
             elif oldchanges.excludecount == len(oldchanges.hunks):
-                self.tv.model().checked[file] = False
+                model.checked[file] = False
                 dels.append(file)
         for file in dels:
             del self.partials[file]
@@ -440,7 +447,7 @@ class StatusWidget(QWidget):
         if self.refthread.wctx is not None:
             assert self.refthread.wstatus is not None
             self.updateModel(self.refthread.wctx, self.refthread.wstatus,
-                             self.refthread.patchecked)
+                             self.refthread.patchecked, self.refthread.amending)
         self.refthread = None
         if len(self.repo[None].parents()) > 1:
             # nuke partial selections if wctx has a merge in-progress
@@ -460,10 +467,11 @@ class StatusWidget(QWidget):
     def canExit(self):
         return not self.isRefreshingWctx()
 
-    def updateModel(self, wctx, wstatus, patchecked):
+    def updateModel(self, wctx, wstatus, patchecked, amending):
         self.tv.setSortingEnabled(False)
-        if self.tv.model():
-            checked = self.tv.model().getChecked()
+        oldtm = self.tv.model()
+        if oldtm:
+            checked = oldtm.getChecked()
         else:
             checked = patchecked
             if self.pats and not checked:
@@ -473,13 +481,13 @@ class StatusWidget(QWidget):
         ms = hglib.readmergestate(self.repo)
         tm = WctxModel(self._repoagent, wctx, wstatus, ms, self.pctx,
                        self.savechecks, self.opts, checked, self,
-                       checkable=self.checkable, defcheck=self.defcheck)
+                       checkable=self.checkable, defcheck=self.defcheck,
+                       amending=amending)
         if self.checkable:
             tm.checkToggled.connect(self.checkToggled)
             tm.checkCountChanged.connect(self.updateCheckCount)
         self.savechecks = True
 
-        oldtm = self.tv.model()
         self.tv.setModel(tm)
         if oldtm:
             oldtm.deleteLater()
@@ -490,7 +498,7 @@ class StatusWidget(QWidget):
             self.updateCheckCount()
 
         # remove non-existent file from partials table because model changed
-        for file in self.partials.keys():
+        for file in list(self.partials):
             if file not in tm.checked:
                 del self.partials[file]
 
@@ -525,7 +533,9 @@ class StatusWidget(QWidget):
     #@pyqtSlot(QModelIndex)
     def onRowDoubleClicked(self, index):
         'tree view emitted a doubleClicked signal, index guarunteed valid'
-        fd = self.tv.model().fileData(index)
+        model = self.tv.model()
+        assert model is not None
+        fd = model.fileData(index)
         if fd.subrepoType():
             self._fileactions.openSubrepo()
         elif fd.mergeStatus() == 'U':
@@ -591,13 +601,22 @@ class StatusWidget(QWidget):
         if model:
             model.checkAll(False)
 
+    def getCheckedAmends(self):
+        files = []
+        model = self.tv.model()
+        if model and model.amending:
+            for f, v in model.getChecked().items():
+                if f in model.amending:
+                    files.append(f)
+        return files
+
     def getChecked(self, types=None):
         model = self.tv.model()
         if model:
             checked = model.getChecked()
             if types is None:
                 files = []
-                for f, v in checked.iteritems():
+                for f, v in checked.items():
                     if f in self.partials:
                         changes = self.partials[f]
                         if changes.excludecount < len(changes.hunks):
@@ -624,7 +643,7 @@ class StatusWidget(QWidget):
     def onSelectionChange(self):
         model = self.tv.model()
         selmodel = self.tv.selectionModel()
-        selfds = map(model.fileData, selmodel.selectedRows())
+        selfds = pycompat.maplist(model.fileData, selmodel.selectedRows())
         self._fileactions.setFileDataList(selfds)
 
     # Disabled decorator because of bug in older PyQt releases
@@ -633,6 +652,7 @@ class StatusWidget(QWidget):
         'Connected to treeview "currentChanged" signal'
         changeselect = self.fileview.isChangeSelectionEnabled()
         model = self.tv.model()
+        assert model is not None
         fd = model.fileData(index)
         fd.load(changeselect)
         if changeselect and not fd.isNull() and not fd.subrepoType():
@@ -641,6 +661,7 @@ class StatusWidget(QWidget):
 
     def _setCheckStateOfSelectedFiles(self, value):
         model = self.tv.model()
+        assert model is not None
         selmodel = self.tv.selectionModel()
         for index in selmodel.selectedRows(COL_PATH):
             model.setData(index, value, Qt.CheckStateRole)
@@ -679,9 +700,10 @@ class StatusThread(QThread):
         self.wctx = None
         self.wstatus = None
         self.patchecked = {}
+        self.amending = set()
 
     def run(self):
-        extract = lambda x, y: dict(zip(x, map(y.get, x)))
+        extract = lambda x, y: dict(zip(x, pycompat.maplist(y.get, x)))
         stopts = extract(('unknown', 'ignored', 'clean'), self.opts)
         patchecked = {}
         try:
@@ -693,28 +715,45 @@ class StatusThread(QThread):
                     # status and commit only pre-check MAR files
                     precheckfn = lambda x: x < 4
                 m = scmutil.match(self.repo[None], self.pats)
-                self.repo.lfstatus = True
-                status = self.repo.status(match=m, **stopts)
-                self.repo.lfstatus = False
+                with lfutil.lfstatus(self.repo):
+                    status = self.repo.status(match=m, **stopts)
                 # Record all matched files as initially checked
                 for i, stat in enumerate(StatusType.preferredOrder):
                     if stat == 'S':
                         continue
                     val = statusTypes[stat]
                     if self.opts[val.name]:
-                        d = dict([(fn, precheckfn(i)) for fn in status[i]])
+                        d = dict([(fn, precheckfn(i))
+                                  for fn in getattr(status, val.name)])
                         patchecked.update(d)
                 wctx = context.workingctx(self.repo, changes=status)
                 self.patchecked = patchecked
             elif self.pctx:
-                self.repo.lfstatus = True
-                status = self.repo.status(node1=self.pctx.p1().node(), **stopts)
-                self.repo.lfstatus = False
+                with lfutil.lfstatus(self.repo):
+                    status = self.repo.status(node1=self.pctx.p1().node(), **stopts)
+                    wstatus = self.repo.status(**stopts)
+
+                # Even though `clean` isn't requested in the status call,
+                # the dirty files in wdir that are clean against p1('.') are in
+                # the clean list.  Those need to be forced to show, so they
+                # aren't excluded from the amend command.
+                self.amending = set(status.clean)
+
+                # For a file that is newly added and then renamed, the status
+                # of wdir against p1 will show nothing (not present to not
+                # present).  Amending in this state breaks the rename, as only
+                # the added file is visible.  Forcing it into the R list will
+                # let is show as R.  Adding it to the amending list would make
+                # it show C, which is consistent with the revert to p1(.) case,
+                # but very confusing for a rename.
+                for r in wstatus.removed:
+                    if r not in self.pctx.p1():
+                        status.removed.append(r)
+
                 wctx = context.workingctx(self.repo, changes=status)
             else:
-                self.repo.lfstatus = True
-                status = self.repo.status(**stopts)
-                self.repo.lfstatus = False
+                with lfutil.lfstatus(self.repo):
+                    status = self.repo.status(**stopts)
                 wctx = context.workingctx(self.repo, changes=status)
             self.wctx = wctx
             self.wstatus = status
@@ -723,11 +762,11 @@ class StatusThread(QThread):
             for s in wctx.substate:
                 if wctx.sub(s).dirty():
                     wctx.dirtySubrepos.append(s)
-        except EnvironmentError, e:
+        except EnvironmentError as e:
             self.showMessage.emit(hglib.tounicode(str(e)))
-        except (error.LookupError, error.RepoError, error.ConfigError), e:
+        except (error.LookupError, error.RepoError, error.ConfigError) as e:
             self.showMessage.emit(hglib.tounicode(str(e)))
-        except error.Abort, e:
+        except error.Abort as e:
             if e.hint:
                 err = _('%s (hint: %s)') % (hglib.tounicode(str(e)),
                                             hglib.tounicode(e.hint))
@@ -750,7 +789,8 @@ class WctxModel(QAbstractTableModel):
     checkToggled = pyqtSignal(str, bool)
 
     def __init__(self, repoagent, wctx, wstatus, ms, pctx, savechecks, opts,
-                 checked, parent, checkable=True, defcheck='MAR!S'):
+                 checked, parent, checkable=True, defcheck='MAR!S',
+                 amending=None):
         QAbstractTableModel.__init__(self, parent)
         self._repoagent = repoagent
         self._pctx = pctx
@@ -759,10 +799,13 @@ class WctxModel(QAbstractTableModel):
         rows = []
         nchecked = {}
         excludes = [f.strip() for f in opts.get('ciexclude', '').split(',')]
+        if amending is None:
+            amending = set()
+
         def mkrow(fname, st):
             ext, sizek = '', ''
             try:
-                mst = fname in ms and ms[fname].upper() or ""
+                mst = fname in ms and pycompat.sysstr(ms[fname].upper()) or ""
                 name, ext = os.path.splitext(fname)
                 sizebytes = wctx[fname].size()
                 sizek = (sizebytes + 1023) // 1024
@@ -775,7 +818,14 @@ class WctxModel(QAbstractTableModel):
             # Currently, having a patch context means it's a qrefresh, so only
             # auto-check files in pctx.files()
             pctxfiles = pctx.files()
-            pctxmatch = lambda f: f in pctxfiles
+
+            def pctxmatch(f):
+                if f in pctxfiles:
+                    return True
+                if f in wctx:  # auto select copy/rename sources too
+                    return wctx[f].copysource() in pctxfiles
+                return False
+
         else:
             pctxmatch = lambda f: True
         if opts['modified']:
@@ -812,19 +862,31 @@ class WctxModel(QAbstractTableModel):
                 rows.append(mkrow(c, 'C'))
         if opts['subrepo']:
             for s in wctx.dirtySubrepos:
-                nchecked[s] = checked.get(s, 'S' in defcheck)
+                nchecked[s] = checked.get(s, 'S' in defcheck and
+                                          s not in excludes)
                 rows.append(mkrow(s, 'S'))
         # include clean unresolved files
         for f in ms:
-            if ms[f] == 'u' and f not in nchecked:
+            if ms[f] == b'u' and f not in nchecked:
                 nchecked[f] = checked.get(f, True)
                 rows.append(mkrow(f, 'C'))
+
+        # In the amend case, files reverted to p1('.') look clean because the
+        # status is calculated against that node for amends.  But they are
+        # really modified.  So force those clean files to show, and to be in
+        # the checked state.
+        for f in amending:
+            if f not in nchecked:
+                nchecked[f] = checked.get(f, True)
+                rows.append(mkrow(f, 'C'))
+
         self.headers = ('*', _('Stat'), _('M'), _('Filename'),
                         _('Type'), _('Size (KB)'))
         self.checked = nchecked
         self.unfiltered = rows
         self.rows = rows
         self.checkable = checkable
+        self.amending = amending
 
     def rowCount(self, parent):
         if parent.isValid():
@@ -1006,7 +1068,7 @@ class WctxModel(QAbstractTableModel):
         # However, for files which have the same status or extension, etc,
         # we want them to be sorted alphabetically (without taking into account
         # the case)
-        # Since Python 2.3 the sort function is guaranteed to be stable.
+        # The sort function is guaranteed to be stable.
         # Thus we can perform the sort in two passes:
         # 1.- Perform a secondary sort by path
         # 2.- Perform a primary sort by the actual column that we are sorting on
@@ -1044,7 +1106,7 @@ class WctxModel(QAbstractTableModel):
         self.layoutAboutToBeChanged.emit()
         self.beginResetModel()
         self.rows = [r for r in self.unfiltered
-                     if unicode(match) in r[COL_PATH_DISPLAY]]
+                     if pycompat.unicode(match) in r[COL_PATH_DISPLAY]]
         self.layoutChanged.emit()
         self.endResetModel()
 
@@ -1134,7 +1196,7 @@ class StatusFilterActionGroup(QObject):
     @pyqtSlot(str)
     def setStatus(self, text):
         """Set the status text"""
-        assert all(c in self._TYPES for c in text)
+        assert all(c in self._TYPES for c in text), repr(text)
         for c in self._TYPES:
             self._actions[c].setChecked(c in text)
 
@@ -1186,7 +1248,7 @@ class StatusDialog(QDialog):
         QTimer.singleShot(0, self.stwidget.refreshWctx)
 
     def linkActivated(self, link):
-        link = unicode(link)
+        link = pycompat.unicode(link)
         if link.startswith('repo:'):
             self._subdialogs.open(link[len('repo:'):])
 
