@@ -38,6 +38,9 @@ import time
 
 from mercurial import (
     error,
+    scmutil,
+    pycompat,
+    util,
 )
 
 from tortoisehg.util import hgversion
@@ -45,18 +48,18 @@ from tortoisehg.util.i18n import agettext as _
 
 testedwith = hgversion.testedwith
 
-class _labeledstr(str):
+class _labeledstr(bytes):
     r"""
-    >>> a = _labeledstr('foo', 'ui.warning')
+    >>> a = _labeledstr(b'foo', b'ui.warning')
     >>> a.packed()
-    '\x01ui.warning\nfoo'
-    >>> _labeledstr(a, 'ui.error').packed()
-    '\x01ui.warning ui.error\nfoo'
-    >>> _labeledstr('foo', '').packed()
-    'foo'
-    >>> _labeledstr('\1foo', '').packed()
-    '\x01\n\x01foo'
-    >>> _labeledstr(a, '') is a  # fast path
+    b'\x01ui.warning\nfoo'
+    >>> _labeledstr(a, b'ui.error').packed()
+    b'\x01ui.warning ui.error\nfoo'
+    >>> _labeledstr(b'foo', b'').packed()
+    b'foo'
+    >>> _labeledstr(b'\1foo', b'').packed()
+    b'\x01\n\x01foo'
+    >>> _labeledstr(a, b'') is a  # fast path
     True
     """
 
@@ -65,33 +68,33 @@ class _labeledstr(str):
             if not l:
                 return s
             if s._label:
-                l = s._label + ' ' + l
-        t = str.__new__(cls, s)
+                l = s._label + b' ' + l
+        t = bytes.__new__(cls, s)
         t._label = l
         return t
 
     def packed(self):
-        if not self._label and not self.startswith('\1'):
-            return str(self)
-        return '\1%s\n%s' % (self._label, self)
+        if not self._label and not self.startswith(b'\1'):
+            return bytes(self)
+        return b'\1%s\n%s' % (self._label, self)
 
 def _packmsgs(msgs, label):
     r"""
-    >>> _packmsgs(['foo'], '')
-    ['foo']
-    >>> _packmsgs(['foo ', 'bar'], '')
-    ['foo bar']
-    >>> _packmsgs(['foo ', 'bar'], 'ui.status')
-    ['\x01ui.status\nfoo bar']
-    >>> _packmsgs(['foo ', _labeledstr('bar', 'log.branch')], '')
-    ['foo ', '\x01log.branch\nbar']
+    >>> _packmsgs([b'foo'], b'')
+    [b'foo']
+    >>> _packmsgs([b'foo ', b'bar'], b'')
+    [b'foo bar']
+    >>> _packmsgs([b'foo ', b'bar'], b'ui.status')
+    [b'\x01ui.status\nfoo bar']
+    >>> _packmsgs([b'foo ', _labeledstr(b'bar', b'log.branch')], b'')
+    [b'foo ', b'\x01log.branch\nbar']
     """
     if not any(isinstance(e, _labeledstr) for e in msgs):
         # pack into single message to avoid overhead of label header and
         # channel protocol; also it's convenient for command-server client
         # to receive the whole message at once.
         if len(msgs) > 1:
-            msgs = [''.join(msgs)]
+            msgs = [b''.join(msgs)]
         if not label:
             # fast path
             return msgs
@@ -101,61 +104,61 @@ def splitmsgs(data):
     r"""Split data to list of packed messages assuming that original messages
     contain no '\1' character
 
-    >>> splitmsgs('')
+    >>> splitmsgs(b'')
     []
-    >>> splitmsgs('\x01ui.warning\nfoo\x01\nbar')
-    ['\x01ui.warning\nfoo', '\x01\nbar']
-    >>> splitmsgs('foo\x01ui.warning\nbar')
-    ['foo', '\x01ui.warning\nbar']
+    >>> splitmsgs(b'\x01ui.warning\nfoo\x01\nbar')
+    [b'\x01ui.warning\nfoo', b'\x01\nbar']
+    >>> splitmsgs(b'foo\x01ui.warning\nbar')
+    [b'foo', b'\x01ui.warning\nbar']
     """
-    msgs = data.split('\1')
+    msgs = data.split(b'\1')
     if msgs[0]:
-        return msgs[:1] + ['\1' + e for e in msgs[1:]]
+        return msgs[:1] + [b'\1' + e for e in msgs[1:]]
     else:
-        return ['\1' + e for e in msgs[1:]]
+        return [b'\1' + e for e in msgs[1:]]
 
 def unpackmsg(data):
     r"""Try to unpack data to original message and label
 
-    >>> unpackmsg('foo')
-    ('foo', '')
-    >>> unpackmsg('\x01ui.warning\nfoo')
-    ('foo', 'ui.warning')
-    >>> unpackmsg('\x01ui.warning')  # immature end
-    ('', 'ui.warning')
+    >>> unpackmsg(b'foo')
+    (b'foo', b'')
+    >>> unpackmsg(b'\x01ui.warning\nfoo')
+    (b'foo', b'ui.warning')
+    >>> unpackmsg(b'\x01ui.warning')  # immature end
+    (b'', b'ui.warning')
     """
-    if not data.startswith('\1'):
-        return data, ''
+    if not data.startswith(b'\1'):
+        return data, b''
     try:
-        label, msg = data[1:].split('\n', 1)
+        label, msg = data[1:].split(b'\n', 1)
         return msg, label
     except ValueError:
-        return '', data[1:]
+        return b'', data[1:]
 
 def _packprompt(msg, default):
     r"""
-    >>> _packprompt('foo', None)
-    'foo\x00'
-    >>> _packprompt(_labeledstr('$$ &Yes', 'ui.promptchoice'), 'y').packed()
-    '\x01ui.promptchoice\n$$ &Yes\x00y'
+    >>> _packprompt(b'foo', None)
+    b'foo\x00'
+    >>> _packprompt(_labeledstr(b'$$ &Yes', b'ui.promptchoice'), b'y').packed()
+    b'\x01ui.promptchoice\n$$ &Yes\x00y'
     """
-    s = '\0'.join((msg, default or ''))
+    s = b'\0'.join((msg, default or b''))
     if not isinstance(msg, _labeledstr):
         return s
     return _labeledstr(s, msg._label)
 
 def unpackprompt(msg):
     """Try to unpack prompt message to original message and default value"""
-    args = msg.split('\0', 1)
+    args = msg.split(b'\0', 1)
     if len(args) == 1:
-        return msg, ''
+        return msg, b''
     else:
         return args
 
 def _fromint(n):
     if n is None:
-        return ''
-    return str(n)
+        return b''
+    return b'%d' % n
 
 def _toint(s):
     if not s:
@@ -164,34 +167,34 @@ def _toint(s):
 
 def _packprogress(topic, pos, item, unit, total):
     r"""
-    >>> _packprogress('updating', 1, 'foo', 'files', 5)
-    'updating\x001\x00foo\x00files\x005'
-    >>> _packprogress('updating', None, '', '', None)
-    'updating\x00\x00\x00\x00'
+    >>> _packprogress(b'updating', 1, b'foo', b'files', 5)
+    b'updating\x001\x00foo\x00files\x005'
+    >>> _packprogress(b'updating', None, b'', b'', None)
+    b'updating\x00\x00\x00\x00'
     """
-    return '\0'.join((topic, _fromint(pos), item, unit, _fromint(total)))
+    return b'\0'.join((topic, _fromint(pos), item, unit, _fromint(total)))
 
 def unpackprogress(msg):
     r"""Try to unpack progress message to tuple of parameters
 
-    >>> unpackprogress('updating\x001\x00foo\x00files\x005')
-    ('updating', 1, 'foo', 'files', 5)
-    >>> unpackprogress('updating\x00\x00\x00\x00')
-    ('updating', None, '', '', None)
-    >>> unpackprogress('updating\x001\x00foo\x00files')  # immature end
-    ('updating', None, '', '', None)
-    >>> unpackprogress('')  # no separator
-    ('', None, '', '', None)
-    >>> unpackprogress('updating\x00a\x00foo\x00files\x005')  # invalid pos
-    ('updating', None, '', '', None)
+    >>> unpackprogress(b'updating\x001\x00foo\x00files\x005')
+    (b'updating', 1, b'foo', b'files', 5)
+    >>> unpackprogress(b'updating\x00\x00\x00\x00')
+    (b'updating', None, b'', b'', None)
+    >>> unpackprogress(b'updating\x001\x00foo\x00files')  # immature end
+    (b'updating', None, b'', b'', None)
+    >>> unpackprogress(b'')  # no separator
+    (b'', None, b'', b'', None)
+    >>> unpackprogress(b'updating\x00a\x00foo\x00files\x005')  # invalid pos
+    (b'updating', None, b'', b'', None)
     """
     try:
-        topic, pos, item, unit, total = msg.split('\0')
+        topic, pos, item, unit, total = msg.split(b'\0')
         return topic, _toint(pos), item, unit, _toint(total)
     except ValueError:
         # fall back to termination
-        topic = msg.split('\0', 1)[0]
-        return topic, None, '', '', None
+        topic = msg.split(b'\0', 1)[0]
+        return topic, None, b'', b'', None
 
 _progressrefresh = 0.1  # [sec]
 
@@ -200,15 +203,12 @@ def _extenduiclass(parcls):
         _lastprogresstopic = None
         _lastprogresstime = 0
 
-        def _writenobuf(self, *args, **opts):
-            label = opts.get('label', '')
-            super(pipeui, self)._writenobuf(*_packmsgs(args, label), **opts)
+        def _writenobuf(self, dest, *args, **opts):
+            label = opts.get('label', b'')
+            super(pipeui, self)._writenobuf(dest, *_packmsgs(args, label),
+                                            **opts)
 
-        def write_err(self, *args, **opts):
-            label = opts.get('label', '')
-            super(pipeui, self).write_err(*_packmsgs(args, label), **opts)
-
-        def prompt(self, msg, default='y'):
+        def prompt(self, msg, default=b'y'):
             fullmsg = _packprompt(msg, default)
             return super(pipeui, self).prompt(fullmsg, default)
 
@@ -216,7 +216,7 @@ def _extenduiclass(parcls):
         def promptchoice(self, prompt, default=0):
             _msg, choices = self.extractchoices(prompt)
             resps = [r for r, _t in choices]
-            prompt = self.label(prompt, 'ui.promptchoice')
+            prompt = self.label(prompt, b'ui.promptchoice')
             r = self.prompt(prompt, resps[default])
             try:
                 return resps.index(r.lower())
@@ -224,9 +224,11 @@ def _extenduiclass(parcls):
                 raise error.Abort(_('unrecognized response: %s') % r)
 
         def getpass(self, prompt=None, default=None):
-            prompt = self.label(prompt or _('password: '), 'ui.getpass')
+            prompt = self.label(prompt or _('password: '), b'ui.getpass')
             return super(pipeui, self).getpass(prompt, default)
 
+        # TODO: progress handler can be extracted to new class, and pass its
+        # instance to scmutil.progress() per makeprogress() call.
         def progress(self, topic, pos, item='', unit='', total=None):
             now = time.time()
             if (topic == self._lastprogresstopic and pos is not None
@@ -240,7 +242,10 @@ def _extenduiclass(parcls):
                 self._lastprogresstopic = topic
             self._lastprogresstime = now
             msg = _packprogress(topic, pos, item, unit, total)
-            self.write_err(msg, label='ui.progress')
+            self.write_err(msg, label=b'ui.progress')
+
+        def makeprogress(self, topic, unit='', total=None):
+            return scmutil.progress(self, self.progress, topic, unit, total)
 
         def label(self, msg, label):
             return _labeledstr(msg, label)
