@@ -7,9 +7,20 @@
 
 import os
 import re
-import cStringIO
-import ConfigParser
-from mercurial import error, util, config as config_mod
+
+from mercurial import (
+    config as config_mod,
+    error,
+    pycompat,
+    util,
+)
+
+from tortoisehg.util import hglib
+
+if pycompat.ispy3:
+    import configparser
+else:
+    import ConfigParser as configparser
 
 try:
     from iniparse import INIConfig
@@ -54,6 +65,9 @@ class _wsortdict(object):
 
     def _logset(self, key, val):
         """Record set operation to log; called also by _wconfig"""
+        if pycompat.ispy3:
+            key = pycompat.sysstr(key)
+            val = hglib.tounicode(val)
         def op(target):
             target[key] = val
         self._log.append(op)
@@ -95,6 +109,8 @@ class _wsortdict(object):
 
     def _logdel(self, key):
         """Record del operation to log"""
+        if pycompat.ispy3:
+            key = pycompat.sysstr(key)
         def op(target):
             try:
                 del target[key]
@@ -145,6 +161,11 @@ class _wconfig(object):
             return self._sections[section]
         except KeyError:
             if self._config[section]:
+                # get around COW behavior introduced by hg c41444a39de2, where
+                # an inner dict may be replaced later on preparewrite(). our
+                # wrapper expects non-empty config[section] instance persists.
+                data = self._config._data
+                data[section] = data[section].preparewrite()
                 self._sections[section] = _wsortdict(self._config[section])
                 return self._sections[section]
             else:
@@ -161,7 +182,10 @@ class _wconfig(object):
         for s in src:
             self[s]._logupdate(src[s])
 
-    def set(self, section, item, value, source=''):
+    def set(self, section, item, value, source=b''):
+        assert isinstance(section, bytes), (section, item, value)
+        assert isinstance(item, bytes), (section, item, value)
+        assert isinstance(value, bytes), (section, item, value)
         self._setconfig(section, item, value, source)
         self[section]._logset(item, value)
 
@@ -195,10 +219,10 @@ class _wconfig(object):
             try:
                 # TODO: optionxformvalue isn't used by INIConfig ?
                 return INIConfig(fp=fp, optionxformvalue=None)
-            except ConfigParser.MissingSectionHeaderError, err:
+            except configparser.MissingSectionHeaderError as err:
                 raise error.ParseError(err.message.splitlines()[0],
                                        '%s:%d' % (err.filename, err.lineno))
-            except ConfigParser.ParsingError, err:
+            except configparser.ParsingError as err:
                 if err.errors:
                     loc = '%s:%d' % (err.filename, err.errors[0][0])
                 else:
@@ -216,13 +240,23 @@ class _wconfig(object):
 
         if fp:
             fp.seek(0)
-            return newini(fp)
-        else:
-            fp = util.posixfile(path, 'rb')
-            try:
+            if pycompat.ispy3:
+                try:
+                    fp_data = pycompat.io.StringIO(hglib.tounicode(fp.read()))
+                    return newini(fp_data)
+                finally:
+                    fp.close()
+            else:
                 return newini(fp)
+        else:
+            fp_data = fp_file = util.posixfile(path, b'rb')
+            if pycompat.ispy3:
+                data = hglib.tounicode(fp_data.read())
+                fp_data = pycompat.io.StringIO(data)
+            try:
+                return newini(fp_data)
             finally:
-                fp.close()
+                fp_file.close()
 
     def _replaylogs(self, ini):
         def getsection(ini, section):
@@ -233,8 +267,11 @@ class _wconfig(object):
                                 getattr(ini, 'new_namespace'))
                 return newns(section)
 
-        for k, v in self._sections.iteritems():
-            v._replaylog(getsection(ini, k))
+        for section, sortdict in self._sections.items():
+            if pycompat.ispy3:
+                section = pycompat.sysstr(section)
+            target = getsection(ini, section)
+            sortdict._replaylog(target)
 
     def __getattr__(self, name):
         return getattr(self._config, name)
@@ -258,10 +295,15 @@ def readfile(path):
 
 def writefile(config, path):
     """Write the given config obj to the specified file"""
-    # normalize line endings
-    buf = cStringIO.StringIO()
-    config.write(buf)
-    data = '\n'.join(buf.getvalue().splitlines()) + '\n'
+    if pycompat.ispy3:
+        buf = pycompat.io.StringIO()
+        config.write(buf)
+        value = hglib.fromunicode(buf.getvalue())
+    else:
+        buf = pycompat.bytesio()
+        config.write(buf)
+        value = buf.getvalue()
+    data = pycompat.oslinesep.join(value.splitlines() + [b''])
 
     if os.name == 'nt':
         # no atomic rename to the existing file that may fail occasionally
@@ -271,7 +313,7 @@ def writefile(config, path):
     else:
         # atomic rename is reliable on Unix
         openfile = util.atomictempfile
-    f = openfile(os.path.realpath(path), 'w')
+    f = openfile(os.path.realpath(path), b'wb')
     try:
         f.write(data)
         f.close()
